/*  Copyright (C) Samuel Thibault <samuel.thibault@fnac.net>
 *
 * This program is free software ; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation ; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY ; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 * along with the program ; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* common.c: communication via the socket */

#include <stdio.h>
#include <stddef.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

/* WriteFile */
/* Writes a buffer to a file */
/* Returns size on success, -1 if failure */
int WriteFile(int fd, const unsigned char *buf, size_t size)
{
 int n;
 int res=0;
 for (n=0;n<size;n+=res)
 {
  res=write(fd,buf+n,size-n);
  if (res==0) {
   fprintf(stderr,"Couldn't write !\n");
   close(fd);
   return -1;
  }
  if ((res<0) && (errno!=EINTR) && (errno!=EAGAIN)) /* EAGAIN shouldn't happen, but who knows... */
  {
   perror("Socket error while writing, can't recover");
   close(fd);	/* for safety */
   return res;
  }
 }
 return size;
}

/* ReadFile */
/* Reads a buffer from a file */
/* Returns the number of bytes read; */
/* Returns 0 if the end of file is reached before size bytes are read; */
/* Returns -1 if a read() system call fails */
int ReadFile(int fd, unsigned char *buf, size_t size)
{
 int n;
 int res=0;
 for (n=0;n<size && res>=0;n+=res)
 {
  res=read(fd,buf+n,size-n);
  if (res==0) {
   fprintf(stderr,"Unexpected end of file !\n");
   close(fd);
   return 0;
  }
  if ((res<0) && (errno!=EINTR) && (errno!=EAGAIN)) /* EAGAIN shouldn't happen, but who knows... */
  {
   perror("Socket error while reading, can't recover");
   close(fd);	/* for safety */
   return -1;
  }
 }
 return size;
}

