
/*  Copyright (C) Samuel Thibault <samuel.thibault@fnac.net>
 *
 * This program is free software ; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation ; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY ; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 * along with the program ; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <termios.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <netdb.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/time.h>
#include <sys/types.h>
#include <netinet/in.h>
#include "common.h"

#define FAIL(msg) do { \
	perror(msg); \
	exit(2); \
} while (0)

#define CONT(msg) { \
	perror(msg); \
	continue; \
}

#define ERR(msg) { \
	perror(msg); \
	break; \
}

char *argv0;

void grr(void) {
	fprintf(stderr,"usage : %s host port\n",argv0);
	fprintf(stderr,"where host is the host where you launched the server, and port, the port number\nit gave you\n");
	exit(1);
}

struct termios oldtio,tio;
int tty;
void cleantty(void) {
	tcsetattr(tty,TCSANOW,&oldtio);
}

int main(int argc,char **argv) {
	int vcs,s;
	int ntty,err;
	char name[16];
	char *hostname,*port;
	struct addrinfo *res,*cur;
	struct addrinfo hints;
	struct {char lines, cols, x, y; } scrn;
	struct {char lines, cols, x, y; } scrnsrv;
	char c;
	static const char c12=12;
	fd_set fds;

	argv0 = argv[0];

	if (argc<=2) grr();
	hostname=argv[1];
	port=argv[2];

	if ((tty=open("/dev/tty0",O_RDONLY|O_NONBLOCK))<0) FAIL("open /dev/tty0");
	if ((ntty=ioctl(tty,TIOCLINUX,&c12))<0) FAIL("ioctl(TIOCLINUX)");
	ntty++;

	printf("getting control of tty%d\n",ntty);
	sprintf(name,"/dev/vcsa%d",ntty);
	if ((vcs=open(name,O_RDWR))<0) FAIL("open of vcsa");

	if ((read(vcs,&scrn,sizeof(scrn)))<sizeof(scrn)) FAIL("reading screen size");

	printf("screen of %ux%u\n",scrn.cols,scrn.lines);

	memset(&hints,0,sizeof(hints));
	hints.ai_family=PF_UNSPEC;
	hints.ai_socktype=SOCK_STREAM;
	if ((err=getaddrinfo(hostname,port,&hints,&res))) {
		fprintf(stderr,"getaddrinfo(%s,%s): %s\n",hostname,port,gai_strerror(err));
		exit(2);
	}
	s=-1;
	for(cur=res;cur;cur=cur->ai_next) {
		if ((s=socket(cur->ai_family,cur->ai_socktype,cur->ai_protocol))<0) CONT("socket");
		if (connect(s,cur->ai_addr,cur->ai_addrlen)<0) CONT("connect");
		break;
	}
	if (!cur) {
		fprintf(stderr,"couldn't find a way to connect !\n");
		exit(3);
	}
	freeaddrinfo(res);
	
	if (tcgetattr(tty,&tio)<0) FAIL("tcgetattr");
	memcpy(&oldtio,&tio,sizeof(tio));
	if (atexit(cleantty)<0) FAIL("atexit");
	tio.c_iflag = 0;
	tio.c_oflag = 0;
	tio.c_cflag = 0;
	tio.c_lflag = 0;
	if (tcsetattr(tty,TCSAFLUSH,&tio)<0) FAIL("tcsetattr");
	while(1) {
		int res;
		FD_ZERO(&fds);
		FD_SET(tty,&fds);
		FD_SET(s,&fds);
		if (select((s>tty?s:tty)+1,&fds,NULL,NULL,NULL)<0) FAIL("select");
		if (FD_ISSET(tty,&fds)) {
			if (read(tty,&c,1)<1) FAIL("reading from tty");
			if (write(s,&c,1)<1) FAIL("write to server");
		}
		if (FD_ISSET(s,&fds)) {
			if ((res=ReadFile(s,(char *) &scrnsrv,sizeof(scrnsrv)))<sizeof(scrnsrv)) FAIL("reading from server");
			else {
				char line[2*scrnsrv.cols];
				int i;
				
				if (lseek(vcs,2,SEEK_SET)<0) FAIL("lseek");
				if (write(vcs,&scrnsrv.x,2)<2) FAIL("writing coords");
				for(i=0;i<scrnsrv.lines&&i<scrn.lines;i++) {
					if (ReadFile(s,line,sizeof(line))<sizeof(line)) FAIL("reading a line");
					if (scrnsrv.cols>=scrn.cols) {
						if (write(vcs,line,2*scrn.cols)<2*scrn.cols) FAIL("writing line");
					} else {
						if (write(vcs,line,2*scrnsrv.cols)<2*scrnsrv.cols) FAIL("writing line");
						lseek(vcs,2*(scrn.cols-scrnsrv.cols),SEEK_CUR);
					}
				}
				for(;i<scrnsrv.lines;i++) if (ReadFile(s,line,sizeof(line))<sizeof(line)) FAIL("reading last lines");
			}
		}
	}
}
