/*  Copyright (C) Samuel Thibault <samuel.thibault@fnac.net>
 *
 * This program is free software ; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation ; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY ; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 * along with the program ; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <netdb.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/time.h>
#include <netinet/in.h>
#include "common.h"

#define FAIL(msg) do { \
	perror(msg); \
	exit(2); \
} while (0)

#define ERR(msg) { \
	perror(msg); \
	break; \
}

char *argv0;

void grr(void) {
	fprintf(stderr,"usage : %s N\n",argv0);
	fprintf(stderr,"where N is the number of the tty you want to get control of\n");
	exit(1);
}

int main(int argc,char **argv) {
	int vcs,tty,s,client;
	int ntty;
	char name[16];
	struct sockaddr_in addr;
	int addrlen;
	int sizescreen;
	struct {char lines, cols, x, y; } scrn;
	char *screen;
	char *screensent;
	char c;
	fd_set fds;
	struct timeval tout;
	int res;
	addrlen=sizeof(addr);

	argv0 = argv[0];

	if (argc<=1) grr();
	ntty=atoi(argv[1]);

	printf("getting control of tty%d\n",ntty);
	sprintf(name,"/dev/vcsa%d",ntty);
	if ((vcs=open(name,O_RDONLY))<0) FAIL("open of vcsa");
	if ((read(vcs,&scrn,sizeof(scrn)))<sizeof(scrn)) FAIL("reading screen size");
	sizescreen=2*scrn.lines*scrn.cols+4;

	if ((screen=malloc(sizescreen))==NULL) FAIL("mallocing screen");
	if ((screensent=malloc(sizescreen))==NULL) FAIL("mallocing screen sent");

	sprintf(name,"/dev/tty%d",ntty);
	if ((tty=open(name,O_RDONLY))<0) FAIL("open of tty");

	if ((s=socket(AF_INET, SOCK_STREAM, 0))<0) FAIL("socket");
	memset(&addr,0,addrlen);
	addr.sin_family=AF_INET;
	if (bind(s,(struct sockaddr *) &addr,addrlen)<0) FAIL("bind");
	if (getsockname(s, (struct sockaddr *) &addr, &addrlen)<0) FAIL("getsockname");
	printf("using port %d\n",ntohs(addr.sin_port));
	if (listen(s,1)<0) FAIL("listen");

	while ((client=accept(s,NULL,NULL))>=0) {
		printf("connection\n");
		while(1) {
			FD_ZERO(&fds);
			FD_SET(client,&fds);
			tout.tv_sec=0;
			tout.tv_usec=50000;
			if (select(client+1,&fds,NULL,NULL,&tout)) {
				if ((res=read(client,&c,1))<1) {
					if (res<0) perror("read from client");
					break;
				}
				if (ioctl(tty,TIOCSTI,&c)<0) {
					if (errno==EIO) {
						if (close(tty)<0) perror("close");
						if ((tty=open(name,O_RDONLY))<0) FAIL("reopen of tty");
						if (ioctl(tty,TIOCSTI,&c)<0) FAIL("ioctl(TIOCSTI)");
					} else {
						
						fprintf(stderr,"ioctl(TIOCSTI,0x%x)\n",c);
						exit(3);
					}
				}
			} else {
				if (lseek(vcs,0,SEEK_SET)<0) FAIL("lseek");
				if (read(vcs,screen,sizescreen)<sizescreen) FAIL("read vcsa");
				if (memcmp(screen,screensent,sizescreen)) {
					if (WriteFile(client,screen,sizescreen)<sizescreen) ERR("write to client");
					memcpy(screensent,screen,sizescreen);
				}
			}
		}
		if (close(client)<0) perror("close");
		printf("disconnection\n");
		*screensent^=1;	/* to make sure refresh will be done for the new client */
	}
	exit(0);
}
